@@ -2,12 +2,12 @@
 local path implementation.
 """
 from contextlib import contextmanager
-import sys, os, re, atexit
+import sys, os, re, atexit, io
 import py
 from py._path import common
 from stat import S_ISLNK, S_ISDIR, S_ISREG
 
-from os.path import normpath, isabs, exists, isdir, isfile, islink
+from os.path import abspath, normpath, isabs, exists, isdir, isfile, islink
 
 iswin32 = sys.platform == "win32" or (getattr(os, '_name', False) == 'nt')
 
@@ -151,11 +151,10 @@ class LocalPath(FSBase):
         elif isinstance(path, py.builtin._basestring):
             if expanduser:
                 path = os.path.expanduser(path)
-            self.strpath = os.path.abspath(normpath(str(path)))
+            self.strpath = abspath(normpath(path))
         else:
             raise ValueError("can only pass None, Path instances "
                              "or non-empty strings to LocalPath")
-        assert isinstance(self.strpath, py.builtin._basestring)
 
     def __hash__(self):
         return hash(self.strpath)
@@ -184,8 +183,8 @@ class LocalPath(FSBase):
         """ return True if 'other' references the same file as 'self'.
         """
         other = getattr(other, "strpath", other)
-        if not os.path.isabs(other):
-            other = os.path.abspath(other)
+        if not isabs(other):
+            other = abspath(other)
         if self == other:
             return True
         if iswin32:
@@ -331,13 +330,15 @@ class LocalPath(FSBase):
         obj.strpath = normpath(strpath)
         return obj
 
-    def open(self, mode='r', ensure=False):
+    def open(self, mode='r', ensure=False, encoding=None):
         """ return an opened file with the given mode.
 
         If ensure is True, create parent directories if needed.
         """
         if ensure:
             self.dirpath().ensure(dir=1)
+        if encoding:
+            return py.error.checked_call(io.open, self.strpath, mode, encoding=encoding)
         return py.error.checked_call(open, self.strpath, mode)
 
     def _fastjoin(self, name):
@@ -369,7 +370,7 @@ class LocalPath(FSBase):
         if isinstance(fil, py.builtin._basestring):
             if not self._patternchars.intersection(fil):
                 child = self._fastjoin(fil)
-                if os.path.exists(child.strpath):
+                if exists(child.strpath):
                     return [child]
                 return []
             fil = common.FNMatcher(fil)
@@ -435,6 +436,24 @@ class LocalPath(FSBase):
         py.error.checked_call(os.mkdir, getattr(p, "strpath", p))
         return p
 
+    def write_binary(self, data, ensure=False):
+        """ write binary data into path.   If ensure is True create
+        missing parent directories.
+        """
+        if ensure:
+            self.dirpath().ensure(dir=1)
+        with self.open('wb') as f:
+            f.write(data)
+
+    def write_text(self, data, encoding, ensure=False):
+        """ write text data into path using the specified encoding.
+        If ensure is True create missing parent directories.
+        """
+        if ensure:
+            self.dirpath().ensure(dir=1)
+        with self.open('w', encoding=encoding) as f:
+            f.write(data)
+
     def write(self, data, mode='w', ensure=False):
         """ write data into path.   If ensure is True create
         missing parent directories.
@@ -682,7 +701,7 @@ class LocalPath(FSBase):
             Note: This is probably not working on plain win32 systems
             but may work on cygwin.
         """
-        if os.path.isabs(name):
+        if isabs(name):
             p = py.path.local(name)
             if p.check(file=1):
                 return p
@@ -701,9 +720,10 @@ class LocalPath(FSBase):
                                  for path in paths]
                 else:
                     paths = py.std.os.environ['PATH'].split(':')
-            tryadd = ['']
+            tryadd = []
             if iswin32:
                 tryadd += os.environ['PATHEXT'].split(os.pathsep)
+            tryadd.append("")
 
             for x in paths:
                 for addext in tryadd:
